import pika

from .rabbit_setting import no_ack
from .rabbit_setting import delivery_mode


class Rabbit:
    def __init__(self,host="localhost", vhost=None):
        self.connection = None
        self.no_ack = no_ack
        self.delivery_mode = delivery_mode
        self.host = host
        self.vhost = vhost

    def connect(self):
        if self.vhost is None:
            self.connection = pika.BlockingConnection(pika.ConnectionParameters(self.host))
        else:
            self.connection = pika.BlockingConnection(pika.ConnectionParameters(host=self.host,virtual_host=self.vhost))

    def disconnect(self):
        self.connection.close()

    def channel(self):
        return self.connection.channel()

    def queue(self,channel,queue_name,durable=True):
        return channel.queue_declare(queue=queue_name,durable=durable)

    def queue_bind(self,channel,exchange,queue,binding_key=''):
        channel.queue_bind(exchange=exchange,
                           queue=queue.method.queue,
                           routing_key=binding_key)


    def publish(self,channel,exchange,routing_key,body, delivery_mode=2):
        channel.basic_publish(exchange=exchange,
                              routing_key=routing_key,
                              body=body,
                              properties=pika.BasicProperties(
                                  delivery_mode=delivery_mode,  # make message persistent
                              ))

    def consume(self,channel,callback,queue,ack=True):
        """
        :param channel: il canale verso il server 
        :param callback: la funzione che gestisce il messaggio in input
        :param queue: il nome della coda da cui leggere
        :param ack: se il server si aspetta un ack prima di scartare il messaggio
        :return: 
        """
        channel.basic_consume(callback,
                              queue=queue,
                              no_ack=not ack)
        print(' [*] Waiting for messages. To exit press CTRL+C')
        channel.start_consuming()

    def num_mess(self,channel,queue):
        return queue.method.message_count

    def get(self,channel,queue,num_mess,ack=True):
        messages = []
        method_frame = None
        queue_name = queue.method.queue
        for x in range(num_mess):
            method_frame, header_frame, body = channel.basic_get(queue_name)
            messages.append(body.decode("utf8"))
            if ack == True:
                channel.basic_ack(method_frame.delivery_tag)

        if    method_frame is None: return messages,None
        return messages, method_frame.delivery_tag

    def ack(self,channel, delivery_tag, multiple=True):
        channel.basic_ack(delivery_tag, multiple=multiple)

    def nack(self,channel, delivery_tag, multiple=True, requeue=True):
        channel.basic_nack(delivery_tag, multiple=multiple, requeue=requeue)

    def exchange(self,channel,exchange_name,exchange_type='direct'):
        channel.exchange_declare(exchange=exchange_name,
                                 exchange_type=exchange_type)

    def check_connection(self):
        return self.connection.is_open



# encoding: utf-8
from ..mongo import Mongo
from datetime import datetime
import time

class MongoChat(Mongo):
    def __init__(self,host, port, db):
        super(MongoChat,self).__init__(host, port, db)
        self.collection_chat = "Chat"
        self.collection_user_social_sid = "Users_Social_Sid"
        self.collection_user_social = "Users_Social"
        self.collection_user_social_chat_room_id = "Users_Social_Chat_Room_{}"

    def set_status(self, user, status):
        selected_collection = self.connection[self.collection_chat]
        tmp = {
            "user": user,
            "status": status
        }
        selected_collection.insert_one(tmp)

    def update_user_sid(self, user_id, sid):
        selected_collection = self.connection[self.collection_user_social_sid]
        selected_collection.update({"_id": user_id}, {"$set": {"sid": sid}}, upsert=True)

    def get_user_sid(self, user_id):
        selected_collection = self.connection[self.collection_user_social_sid]
        sid = selected_collection.find_one({"_id": user_id}, {"sid": 1})
        if sid:
            return sid["sid"]
        return sid

    def update_status(self, user_id, status):
        selected_collection = self.connection[self.collection_user_social]
        selected_collection.update({"_id": user_id}, {"$set": {"status": status}})

    def register_user(self, user_id):
        selected_collection = self.connection[self.collection_user_social]
        selected_collection.insert_one({"_id": user_id, "friends": [], "status": 0})

    def add_friend(self, user_id, friend_user_id):
        selected_collection = self.connection[self.collection_user_social]
        selected_collection.update({"_id": user_id}, {"$addToSet": {"friends": friend_user_id}})

    def remove_friend(self, user_id, friend_user_id):
        selected_collection = self.connection[self.collection_user_social]
        selected_collection.update({"_id": user_id}, {"$pull": {"friends": friend_user_id}})

    def get_social_user(self, user_id):
        selected_collection = self.connection[self.collection_user_social]
        user = selected_collection.find_one({"_id": user_id})
        return user

    def get_friends_list(self, user_id):
        selected_collection = self.connection[self.collection_user_social]
        friends = selected_collection.find_one({"_id": user_id}, {"friends": 1})
        if friends:
            return friends["friends"]
        return []

    def get_user_id_from_sid(self, sid):
        selected_collection = self.connection[self.collection_user_social_sid]
        user_id = selected_collection.find_one({"sid": sid})
        if user_id:
            return user_id["_id"]
        return user_id

    def store_chat_msg(self, room_id, writer, msg, timestamp):
        collection = self.collection_user_social_chat_room_id.format(room_id)
        selected_collection = self.connection[collection]
        ts = int(timestamp)
        dt = datetime.fromtimestamp(timestamp).strftime('%Y-%m-%d %H:%M:%S')
        selected_collection.insert_one({"_id": ts, "writer": writer, "msg": msg, "date": dt})

    def get_chat_msg(self, room_id, num_msg, last_msg_id):
        collection = self.collection_user_social_chat_room_id.format(room_id)
        selected_collection = self.connection[collection]
        if last_msg_id == 0:
            messages = selected_collection.find().limit(num_msg).sort("_id", -1)
        else:
            messages = selected_collection.find({"_id":{"$lt": last_msg_id}}).limit(num_msg).sort("_id", -1)
        return messages


from pymongo import MongoClient

class MongoSocial(object):
    def __init__(self, host, port, db):
        self.host       = host
        self.port       = port
        self.db         = db

        self.client     = None
        self.connection = None
        self.connect()

    def connect(self):
        self.client     = MongoClient(self.host, self.port)
        self.connection = self.client[self.db]


    def write_social_reccomendations(self, user_id, reccomendations):
        selected_collection = self.connection['Social']
        result = selected_collection.update({"_id": user_id},reccomendations,upsert=True)
        return result

    def write_social_logs(self, user_id, logs):
        selected_collection = self.connection['SocialLogs']
        result = selected_collection.update({"_id": user_id}, logs, upsert=True)
        return result


    def write_social_normalization(self, user_id, normalization):
        selected_collection = self.connection['SocialNormalizations']
        result = selected_collection.update({"_id": user_id}, normalization, upsert=True)
        return result



from ..mongo import Mongo
from pymongo import ReturnDocument

class MongoAffinity(Mongo):
    def __init__(self,host, port, db):
        super(MongoAffinity,self).__init__(host, port, db)


    def update_affinity(self, cluster_info, category, clustering_type):
        collection_name = "AffinityUsers"
        selected_collection = self.connection[collection_name]
        updated = selected_collection.update_one({'category':category}, {"$set":{clustering_type: cluster_info}}, upsert=True)



# encoding: utf-8
from ..mongo import Mongo
import operator

class MongoReccomender(Mongo):
    def __init__(self,host, port, db):
        super(MongoReccomender,self).__init__(host, port, db)

    def add_user_raccomandations(self, user_id, new_raccomandations, category):
        category = category.replace(" ", "")
        collection_name = "Raccomandations_"+category
        selected_collection = self.connection[collection_name]
        for id_news, weight in new_raccomandations.iteritems():
            res = selected_collection.update({'_id': str(user_id)}, {'$set': {'raccomandations.'+str(id_news): weight}}, upsert=True)


    def remove_user_raccomandation(self, user_id, news_id, category):
        category = category.replace(" ", "")
        collection_name = "Raccomandations_"+category
        selected_collection = self.connection[collection_name]
        selected_collection.update({'_id': str(user_id)}, {'$unset': {'raccomandations.'+str(news_id): ""}})


    def add_user_history_raccomandation(self, user_id, news_id, category, weight = 0):
        category = category.replace(" ", "")
        collection_name = "Raccomandations_"+category
        selected_collection = self.connection[collection_name]
        raccomandations = selected_collection.find({'_id': str(user_id)},{'raccomandations.'+str(news_id):1})
        if raccomandations and raccomandations.count()!=0:
            racc = raccomandations[0]['raccomandations']
            if racc:
                weight = racc[str(news_id)]
        selected_collection.update({'_id': str(user_id)}, {'$set': {'history_raccomandations.'+str(news_id): weight}}, upsert=True)


    # funzione che azzera il campo tot della collezione newsVotes per una categoria
    def reset_count_vote(self, category_id):
        collection_name = "News"
        selected_collection = self.connection[collection_name]
        cat_news = selected_collection.find({'category_id': category_id}, {'_id': 1})
        for news in cat_news:
            collection_name = "NewsVotes"
            selected_collection = self.connection[collection_name]
            news_id = news['_id']
            selected_collection.update({'_id': news_id},{"$set":{"tot":0}})


    def get_reccomendations_for_user(self, user_id, category_id_list, num_recs=10, order=0):
        # order = 0 per score raccomandazione
        # order = 1 per tempo (id news)
        collection_name = "CategoriesReccomender"
        selected_collection = self.connection[collection_name]
        result = selected_collection.find({ "_id" : { "$in": category_id_list } })
        #{ "_id" : 11, "name" : "Attualità", "reccomendations" : "Raccomandations_Attualità" }
        #{ "_id" : 13, "name" : "Sport" }
        #{ "_id" : 14, "name" : "Viaggi" }
        #{ "_id" : 15, "name" : "Arte e Cultura" }
        #{ "_id" : 16, "name" : "Intrattenimento", "reccomendations" : "Raccomandations_Intrattenimento" }

        all_news = []
        block_category_news = []
        reccomendations = []
        for elem in result:
            if ("name" not in elem):
                continue
                #return -1

            category_id = int(elem["_id"])
            category_name = elem["name"]
            collection_reccomendation = "Raccomandations_" + category_name

            selected_collection = self.connection[collection_reccomendation]

            tmp_rec = selected_collection.find_one({'_id':str(user_id)})
            if tmp_rec is None:
                #o non esiste la collezione Raccomandations_{categoria} o l'utente non esiste nella collezione
                continue

            if "raccomandations" not in tmp_rec:
                continue
                #return -1

            recs_for_one_category = tmp_rec["raccomandations"]

            recs_for_one_category_cast = [(int(key), recs_for_one_category[key]) for key in recs_for_one_category]

            sorted_recs_for_one_category = []
            if order == 0:
                #sorted_recs_for_one_category = sorted(recs_for_one_category.items(), key=operator.itemgetter(1), reverse=True)[0:int(num_recs)]
                sorted_recs_for_one_category = sorted(recs_for_one_category_cast, key=operator.itemgetter(1),reverse=True)
            elif order == 1:
                #sorted_recs_for_one_category = sorted(recs_for_one_category.items(), key=operator.itemgetter(0),reverse=True)[0:int(num_recs)]
                sorted_recs_for_one_category = sorted(recs_for_one_category_cast, key=operator.itemgetter(0),reverse=True)

            all_news = all_news + sorted_recs_for_one_category[0:int(num_recs)]
            block_category_news.append(sorted_recs_for_one_category[0:int(num_recs)])



        # sorted_all_news = []
        # if order == 0:
        #     sorted_all_news = sorted(all_news, key=operator.itemgetter(1), reverse=True)
        # elif order == 1:
        #     sorted_all_news = sorted(all_news, key=operator.itemgetter(0), reverse=True)
        # list_id_news = [int(elem_[0]) for elem_ in sorted_all_news]
        # # tmp = {
        # #     "category_id":category_id,
        # #     "category_name": category_name,
        # #     "reccomendations": sorted_recs_for_one_category,
        # #     "list_ids": list_id_news
        # # }
        # # reccomendations.append(tmp)
        #
        # reccomendations = {
        #     "reccomendations": sorted_all_news,
        #     "list_ids": list_id_news
        # }
        # #return reccomendations

        # Ordino le news in modo da visualizzarne una per categoria
        sorted_all_news = []
        for i in range(0, int(num_recs)):
            for block_news in block_category_news:
                try:
                    temp_news = block_news[i]
                    sorted_all_news.append(temp_news)
                except IndexError:
                    continue

        list_id_news = [int(elem_[0]) for elem_ in sorted_all_news]

        reccomendations = {
            "reccomendations": sorted_all_news,
            "list_ids": list_id_news
        }
        #return reccomendations

        return reccomendations


    def update_users_for_push_notifications(self, users):
        collection_name = "PushReccomendations"
        selected_collection = self.connection[collection_name]
        selected_collection.update({}, {"$addToSet": {"users": {"$each": users}}}, upsert=True)

    def clean_users_for_push_notifications(self):
        collection_name = "PushReccomendations"
        selected_collection = self.connection[collection_name]
        selected_collection.update({}, {"$set": {"users": []}}, upsert=True)

    def get_users_for_push_notifications(self):
        collection_name = "PushReccomendations"
        selected_collection = self.connection[collection_name]
        result = selected_collection.find_one({})
        return result

    def get_all_categories(self):
        selected_collection = self.connection['CategoriesReccomender']
        return selected_collection.find()

    # def insert_category(self, id, name, enabled):
    #     selected_collection = self.connection['CategoriesReccomender']
    #     selected_collection.update_one({"_id":int(id)}, {"$set": {"name":name, "enabled": int(enabled)}},upsert=True)
    #


    def insert_category(self, id, name, enabled,status=None):
        selected_collection = self.connection['CategoriesReccomender']
        selected_collection.update_one({"_id":int(id)}, {"$set": {"name":name, "enabled": int(enabled), "status":0}},upsert=True)

    def update_category(self, id, name, enabled, status=None):
        selected_collection = self.connection['CategoriesReccomender']
        selected_collection.update_one({"_id":int(id)}, {"$set": {"name":name, "enabled": int(enabled), "status":1}},upsert=True)

    def get_reccomendations_for_user_notifications(self, user_id, category_id_list, num_recs=10, order=0):
        # order = 0 per score raccomandazione
        # order = 1 per tempo (id news)
        collection_name = "CategoriesReccomender"
        selected_collection = self.connection[collection_name]
        result = selected_collection.find({ "_id" : { "$in": category_id_list } })

        all_news = []
        block_category_news = []
        reccomendations = []
        for elem in result:
            if ("name" not in elem):
                continue
                #return -1

            category_id = int(elem["_id"])
            category_name = elem["name"]
            collection_reccomendation = "Raccomandations_" + category_name

            selected_collection = self.connection[collection_reccomendation]

            tmp_rec = selected_collection.find_one({'_id':str(user_id)})
            if tmp_rec is None:
                #o non esiste la collezione Raccomandations_{categoria} o l'utente non esiste nella collezione
                continue

            if "raccomandations" not in tmp_rec:
                continue
                #return -1

            recs_for_one_category = tmp_rec["raccomandations"]

            recs_for_one_category_cast = [(int(key), recs_for_one_category[key]) for key in recs_for_one_category]

            sorted_recs_for_one_category = []
            if order == 0:
                #sorted_recs_for_one_category = sorted(recs_for_one_category.items(), key=operator.itemgetter(1), reverse=True)[0:int(num_recs)]
                sorted_recs_for_one_category = sorted(recs_for_one_category_cast, key=operator.itemgetter(1),reverse=True)
            elif order == 1:
                #sorted_recs_for_one_category = sorted(recs_for_one_category.items(), key=operator.itemgetter(0),reverse=True)[0:int(num_recs)]
                sorted_recs_for_one_category = sorted(recs_for_one_category_cast, key=operator.itemgetter(0),reverse=True)

            all_news = all_news + sorted_recs_for_one_category[0:int(num_recs)]
            block_category_news.append(sorted_recs_for_one_category[0:int(num_recs)])


        # Ordino le news in modo da visualizzarne una per categoria
        sorted_all_news = []
        for i in range(0, int(num_recs)):
            for block_news in block_category_news:
                try:
                    temp_news = block_news[i]
                    sorted_all_news.append(temp_news)
                except IndexError:
                    continue

        list_id_news = [int(elem_[0]) for elem_ in sorted_all_news]
        #print("Lista raccomandazioni: " + str(list_id_news))

        # Prendo le ultime notifiche inviate all'utente
        most_recommended = None
        collection_notifications = "PushReccomendationsHistory"
        selected_collection = self.connection[collection_notifications]
        user_notifications = selected_collection.find_one({'_id': str(user_id)})

        if (user_notifications is None) or ("notifications" not in user_notifications):
            most_recommended = list_id_news[0]

        else:
            for id_news in list_id_news:
                if id_news not in user_notifications["notifications"]:
                    most_recommended = id_news
                    break

        if most_recommended:
            #print("Most recommended: " + str(most_recommended))
            selected_collection.update({"_id": str(user_id)}, {"$push": {"notifications": {"$each": [most_recommended], "$slice": -20}}}, upsert=True)


        return most_recommended



from ..mongo import Mongo

import datetime

class MongoWebApp(Mongo):
    def __init__(self,host, port, db):
        super(MongoWebApp,self).__init__(host, port, db)


    def get_news_reccomandations2(self, user_id,category):
        category = category.replace(" ", "")
        collection_name = "Raccomandations_" + category
        racc_list = []
        try:
            selected_collection = self.connection[collection_name]
            raccomandations = selected_collection.find_one({'_id': str(user_id)}, {'raccomandations': 1})['raccomandations']

            selected_collection = self.connection["News"]
            for key in raccomandations:
                id_news = key
                weight = raccomandations[key]
                racc_news = {}
                news = selected_collection.find_one({"_id": int(id_news)})
                #import pdb
                #pdb.set_trace()
                racc_news["_id"] = id_news
                racc_news["weight"] = weight
                #print(str(news)+" id:"+id_news+" weight:"+str(weight))
                racc_news["category_title"] = news["category_title"]
                racc_news["news_source"] = news["news_source"]
                racc_news["url"] = news["url"]
                racc_news["article"] = news["article"]
                racc_news["title"] = news["title"]
                racc_list.append(racc_news)
        except Exception as e:
            print(str(e))
        return racc_list

    def get_news_reccomandations(self, user_id):
        selected_collection = self.connection["Raccomandations_Sport"]
        result = selected_collection.find_one({"_id": str(user_id)},{"raccomandations":1})
        selected_collection = self.connection["News"]
        racc_list = []
        raccomandations = result["raccomandations"]

        for key in raccomandations:
            id_news = key
            weight = raccomandations[key]
            racc_news = {}
            news = selected_collection.find_one({"_id": int(id_news)})
            racc_news["_id"] = id_news
            racc_news["weight"] = weight
            #print(str(news)+" id:"+id_news+" weight:"+str(weight))
            racc_news["category_title"] = news["category_title"]
            racc_news["news_source"] = news["news_source"]
            racc_news["url"] = news["url"]
            racc_news["article"] = news["article"]
            racc_news["title"] = news["title"]
            racc_list.append(racc_news)

        return racc_list
    # def get_news_reccomandations(self, user_id):
    #     selected_collection = self.connection["Users"]
    #     result = selected_collection.find_one({"_id": str(user_id)},{"raccomandations":1})
    #     selected_collection = self.connection["News"]
    #     racc_list = []
    #     for id_news, weight in result["raccomandations"].iteritems():
    #         racc_news = {}
    #         news = selected_collection.find_one({"_id": int(id_news)})
    #         racc_news["_id"] = id_news
    #         racc_news["weight"] = weight
    #         #print(str(news)+" id:"+id_news+" weight:"+str(weight))
    #         racc_news["category_title"] = news["category_title"]
    #         racc_news["news_source"] = news["news_source"]
    #         racc_news["url"] = news["url"]
    #         racc_news["article"] = news["article"]
    #         racc_news["title"] = news["title"]
    #         racc_list.append(racc_news)
    #
    #     return racc_list


    def get_news_for_learning(self, category, user_id):
        id_news_for_learning = []
        news_for_learning = []

        selected_collection = self.connection["ControllerCurrent"]
        result = selected_collection.find_one({}, {category: 1})

        id_current_jobs = result[category]
        if id_current_jobs != 0:
            pass

        selected_collection = self.connection["Indexes"]
        result = selected_collection.find_one({"_id": id_current_jobs}, {"id_to_idcorpus":1})

        news_in_current_model = result["id_to_idcorpus"]

        selected_collection = self.connection["NewsVotes"]
        for elem in news_in_current_model:
            id_news = news_in_current_model[elem]
            vote_register = selected_collection.find_one({"_id": id_news}, {"register":1})

            if vote_register is not None and str(user_id) in vote_register["register"]:
                continue
            else:
                id_news_for_learning.append(id_news)

        selected_collection = self.connection["News"]
        for elem in id_news_for_learning:
            result = selected_collection.find_one({"_id":elem}, {"article":1, "category_title":1, "title":1, "url":1, "_id":1})
            news_for_learning.append(result)

        return news_for_learning


    def get_history_racc(self, category, user_id):
        category = category.replace(" ", "")
        collection_name = "Raccomandations_"+category
        selected_collection = self.connection[collection_name]
        result = selected_collection.find_one({'_id': str(user_id)}, {'history_raccomandations':1})#['history_raccomandations']
        if result == None:
            return result
        selected_collection = self.connection["News"]
        racc_list = []
        if "history_raccomandations" not in result:
            return None
        raccomandations = result["history_raccomandations"]
        for key in raccomandations:
            id_news = key
            weight = raccomandations[key]
            racc_news = {}
            news = selected_collection.find_one({"_id": int(id_news)})
            racc_news["_id"] = id_news
            racc_news["weight"] = weight
            #print(str(news)+" id:"+id_news+" weight:"+str(weight))
            racc_news["category_title"] = news["category_title"]
            racc_news["news_source"] = news["news_source"]
            racc_news["url"] = news["url"]
            racc_news["article"] = news["article"]
            racc_news["title"] = news["title"]
            racc_list.append(racc_news)

        return racc_list
        #return result


    def get_voted_news(self, category, user_id):
        selected_collection = self.connection["News"]
        news_in_category = selected_collection.find({'category_title': category}, {'title':1, 'article':1})
        voted_news = []
        for news in news_in_category:
            news_id = news['_id']
            news_title = news['title']
            news_article = news['article']
            #print(str(news_id))
            selected_collection = self.connection["NewsVotes"]
            news_register = selected_collection.find({'_id':news_id},{'register.'+str(user_id):1})
            if news_register.count() != 0:
                register = news_register[0]['register']
                if register: # news votata
                    news_vote = register[str(user_id)]
                    elem = {
                            "_id": str(news_id),
                            "vote": news_vote,
                            "title": news_title,
                            "article": news_article
                            }
                    voted_news.append(elem)

        return voted_news


    # Funzione che recupera le news degli ultimi due giorni relative ad una categoria
    def get_last_news(self, category):
        selected_collection = self.connection["News"]
        last_news = selected_collection.find({"category_title":category}, {"_id":1, "article":1, "category_title":1, "title":1, "url":1, "date":1}).limit(300).sort('_id', -1)
        selected = []

        now = datetime.datetime.now()
        yesterday = now - datetime.timedelta(days=1)
        today_year = now.year
        today_month = now.month
        today_day = now.day
        yesterday_year = yesterday.year
        yesterday_month = yesterday.month
        yesterday_day = yesterday.day

        for t in last_news:
            date = t["date"]
            article = t["article"]
            if article is None:
                continue
            if (date[0]==today_year and date[1]==today_month and date[2]==today_day) or (date[0]==yesterday_year and date[1]==yesterday_month and date[2]==yesterday_day):
                selected.append(t)

        # se non ci sono notizie negli ultimi due giorni, prendo le notizie dell'ultimo giorno di attivita'
        if len(selected) == 0:
            last_news = selected_collection.find({"category_title":category}, {"_id":1, "article":1, "category_title":1, "title":1, "url":1, "date":1}).limit(300).sort('_id', -1)
            today_year = last_news[0]["date"][0]
            today_month = last_news[0]["date"][1]
            today_day = last_news[0]["date"][2]

            for t in last_news:
                date = t["date"]
                article = t["article"]
                if article is None:
                    continue
                if (date[0]==today_year and date[1]==today_month and date[2]==today_day):
                    selected.append(t)

        return selected
        #last_news = selected_collection.find_one({"date":elem}, {"article":1, "category_title":1, "title":1, "url":1, "_id":1})


class Receiver:
    def __init__(self,rabbit_instance,queue_name,exchange_name=None):
        self.rabbit = rabbit_instance
        if self.rabbit.check_connection() == False:
            print(' [*] Connection is closed. Reopening connection...')
            self.rabbit.connect()

        self.queue_name = queue_name
        self.channel = self.rabbit.channel()
        if exchange_name is not None:
            self.rabbit.exchange(self.channel,exchange_name,"direct")

        # da eliminare
        list_exchange = ["controller", "news", "parser_error", "reccomender", "rss", "rss_error", "teacher","watcher","metric","social"]
        for ex in list_exchange:
            self.rabbit.exchange(self.channel, ex, "direct")

        self.queue = self.rabbit.queue(self.channel, queue_name)

    def receiver_callback(self,channel, method, properties, body):
        print(" [x] Received %r" % body)
        channel.basic_ack(delivery_tag=method.delivery_tag)

    def bind(self,exchange_name,binding_key):
        self.exchange_name = exchange_name
        self.rabbit.queue_bind(self.channel,exchange_name,self.queue,binding_key)

    def consume(self,callback,ack=True):
        self.rabbit.consume(channel=self.channel, callback=callback, queue=self.queue_name, ack=ack)

    def num_mess(self):
        return self.rabbit.num_mess(self.channel, self.queue)

    def get(self, num_mess, ack=True):
        return self.rabbit.get(self.channel, self.queue, num_mess, ack)

    def ack(self,delivery_tag, multiple=True):
        # if self.rabbit.connection.is_closed:
        #     self.rabbit.connect()
        #     self.channel = self.rabbit.channel()
        self.rabbit.ack(self.channel,delivery_tag,multiple)

    def nack(self,delivery_tag, multiple=True, requeue=True):
        # if self.rabbit.connection.is_closed:
        #     self.rabbit.connect()
        #     self.channel = self.rabbit.channel()
        self.rabbit.nack(self.channel,delivery_tag,multiple,requeue)

    def exchange(self,exchange_name,exchange_type):
        self.rabbit.exchange(self.channel,exchange_name,exchange_type)


# encoding: utf-8
import traceback
from ..mongo import Mongo

class MongoSocialWorker(Mongo):
    def __init__(self,host, port, db):
        super(MongoSocialWorker,self).__init__(host, port, db)
        self.collection_feed_room = "Users_Social_Feed_Room"
        self.collection_feed = "User_Feed_{}"


    def set_feed_room_id(self, user_id, feed_room_id):
        try:
            selected_collection = self.connection[self.collection_feed_room]
            selected_collection.update({"_id": user_id}, {"$set": {"feed_room_id": feed_room_id}}, upsert=True)
            return 1
        except Exception as e:
            traceback.print_exc()
            return None


    def get_feed_room_id(self, user_id):
        try:
            selected_collection = self.connection[self.collection_feed_room]
            result = selected_collection.find_one({"_id": int(user_id)}, {"feed_room_id":1})
            if result is not None and "feed_room_id" in result:
                return result["feed_room_id"]
            else:
                return None
        except Exception as e:
            traceback.print_exc()
            return None


    def get_feed_room_ids(self, list_user_id):

        try:
            selected_collection = self.connection[self.collection_feed_room]

            dict_result = {id: None for id in list_user_id}
            result = selected_collection.find({"_id": {"$in": list_user_id}})
            for elem in result:
                if "feed_room_id" in elem:
                    id = elem["_id"]
                    dict_result[id] = elem["feed_room_id"]

            return dict_result
        except Exception as e:
            traceback.print_exc()
            return None

    def write_post(self, data, user_id):
        selected_collection = self.connection[self.collection_feed.format(user_id)]
        selected_collection.insert(data)


from setuptools import setup

setup(
    name = "justpith",
    version = "1.0.4.32",
    author = "Antonio Caristia",
    author_email = "a.caristia@gmail.com",
    description = ("Core code for justpith infrastructure"),
    license = "GPL",
    keywords = "justpith's core",
    url = "http://packages.python.org/justpith",
    packages=['justpith','justpith.rabbit', 'justpith.mongo','justpith.mongo.teacher', 'justpith.mongo.factorizer', 'justpith.mongo.common', 'justpith.mongo.controller', 'justpith.mongo.reccomender', 'justpith.mongo.web_app', 'justpith.mongo.news_consumer', 'justpith.mongo.chat', 'justpith.mongo.social_worker','justpith.logger', 'justpith.util','justpith.social', 'justpith.mongo.affinity'],
    install_requires=[
          'pika','pymongo','fluent-logger','nltk','stop-words','PyYAML'
    ],
    include_package_data=True,
    zip_safe=False
)
